using System;
using System.IO;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Linq;
using System.Text.RegularExpressions;
using System.Data.SQLite;
//#Include:System.Data.dll
//#Include:System.Data.SQLite.dll

namespace PandoraBox
{
    public class CBIMAGEUniqPlugin : IPluginMacro
    {
        Regex _block = new Regex(@"{PUNIQ(GOOGLE|BING|YANDEX)IMAGE(?<mp>#\d+)*}(.*?){/PUNIQ\1IMAGE}", RegexOptions.Compiled | RegexOptions.Singleline);
        Regex _customBlock = new Regex(@"{PUNIQMYCB\(([^)]+)\)(?<mp>#\d+)*}(.*?){/PUNIQMYCB}", RegexOptions.Compiled | RegexOptions.Singleline);

        public string Execute(string template, PluginMacroArgs args)
        {
            template = Load(template, _block, args);
            template = Load(template, _customBlock, args);
            return template;
        }

        private string Load(string template, Regex regex, PluginMacroArgs args)
        {
            return regex.Replace(template, delegate (Match match)
            {
                var type = match.Value.Contains("MYCB")
                    ? match.Groups[1].Value + ".Directory"
                    : match.Groups[1].Value + ".Images.Directory";
                String typePath = GetIniParameter(type, args);
                bool dbMode = GetIniParameter("Main.UseDatabase", args) == "1";

                var inner = match.Groups[2].Value;
                //получаем текущий ключ
                String key = CleanKey(GetKey(args, match));

                //получаем список использованных элементов
                List<string> usedImages = args.Context[typePath + "UniqImgList"] as List<string>;
                if (usedImages == null)
                    args.Context[typePath + "UniqImgList"] = usedImages = new List<String>();

                //получаем сохраненный ключ контекста
                string oldKey = args.Context[typePath + "UniqImgKey"] as string;
                if (oldKey == null)
                    args.Context[typePath + "UniqImgKey"] = oldKey = "";

                //если ключ поменялся, 
                if (key != oldKey)
                {
                    usedImages.Clear();
                    args.Context[typePath + "UniqImgKey"] = key;
                    if (!dbMode)
                    {
                        String file = Path.Combine(GetIniParameter("Main.DataFolder", args), typePath, GetSub(key.Trim()), key);
                        //читаем строки
                        if (File.Exists(file))
                            usedImages.AddRange(File.ReadAllLines(file).Where(x => x.Length > 0));
                    }
                    else
                    {
                        usedImages.AddRange(GetDBLines(args, GetIniParameter("Main.DataFolder", args), typePath.Replace(".", "_"), key));
                    }
                }

                if (usedImages.Count == 0) return "";
                var idx = Rnd.Next(0, usedImages.Count - 1);
                var image = usedImages[idx];
                usedImages.RemoveAt(idx);
                return inner.Replace("[SRC]", image);
            });
        }

        private string GetKeysFile(PluginMacroArgs args)
        {
            return args.Context["$dbname"] as String ?? "content.db";
        }

        private string[] GetDBLines(PluginMacroArgs args, String dbDir, String table, String key)
        {
            //connect
            var contextKey = "SQLiteConnect";
            SQLiteConnection connection = args.Context[contextKey] as SQLiteConnection;
            if (connection == null)
            {
                connection = new SQLiteConnection("Data Source = \"" + dbDir + "/" + GetKeysFile(args) + "\"; Journal Mode=Off;");
                connection.Open();
                args.Context[contextKey] = connection;
            }

            //reader command
            contextKey = "SQLiteCommand" + table;
            SQLiteCommand reader = args.Context[contextKey] as SQLiteCommand;
            if (reader == null)
            {
                reader = new SQLiteCommand("select content from " + table + " where key = :key", connection);
                reader.Parameters.Add(new SQLiteParameter("key", ""));
                args.Context[contextKey] = reader;
                if (!TableExists(table, connection)) reader.CommandText = null;
            }
            if (reader.CommandText == null) return new string[0]; //no table

            reader.Parameters[0].Value = key;
            var content = (reader.ExecuteScalar() as string) ?? "";
            return content.Split(new[] { '\n' }, StringSplitOptions.RemoveEmptyEntries);
        }

        private bool TableExists(String table, SQLiteConnection connection)
        {
            return new SQLiteCommand("SELECT count(*) FROM sqlite_master WHERE name='" + table + "'", connection).ExecuteScalar().ToString() == "1";
        }

        private string GetIniParameter(String name, PluginMacroArgs args)
        {
            String root = args.Context[name] as String;
            if (root == null)
            {
                root = File.ReadAllLines("ContentBox.ini")
                    .FirstOrDefault(x => x.StartsWith(name + "=", StringComparison.InvariantCultureIgnoreCase));
                root = root != null ? root.Split('=')[1] : "";
                args.Context[name] = root;
            }
            return root;
        }

        private string GetSub(string key)
        {
            return key.Length <= 2 ? key : key.Substring(0, 2).Trim();
        }

        private string GetKey(PluginMacroArgs args, Match match)
        {
            int keyIndex;
            string pos = null;
            var result = args.Key;
            if (args.KeyParts != null && (pos = match.Groups["mp"].Value) != "" && (keyIndex = int.Parse(pos.Trim('#'))) > 0)
                result = args.KeyParts[keyIndex > args.KeyParts.Length ? 0 : keyIndex - 1];
            return result;
        }

        private string CleanKey(String key)
        {
            String key1 = key;
            key1 = String.Join("", key1.Split(Path.GetInvalidPathChars()));
            key1 = String.Join("", key1.Split(Path.GetInvalidFileNameChars()));
            return key1;
        }

        public ushort Level { get { return 5; } }
        public String Name { get { return "CBIMAGEUniqPlugin"; } }
    }
}